/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef _QUBFITS_H
#define _QUBFITS_H

//
// Define API decoration for direct importing of DLL references.
//

#if defined(_WIN32)
#define DLLEXPORT __declspec(dllexport)
#define DLLIMPORT __declspec(dllimport)
#else
#define DLLEXPORT
#define DLLIMPORT
#endif

#if !defined(_QUBFITS_PY_)
#define QUBFITS_PY_API DLLIMPORT
#else
#define QUBFITS_PY_API DLLEXPORT
#endif

#include <Python.h>
#include <qubtree_node.h>
#include <qubfits.h>

///

typedef struct {
  PyObject_HEAD
  IQUB_Curve *impl;
  bool owned;
  void *events;
} qubfits_curve;

typedef struct {
  PyObject_HEAD
  IQUB_Fitter *impl;
  bool owned;
  void *events;
} qubfits_fitter;

#ifdef __cplusplus    /*`__cplusplus' is #defined iff compiler is C++*/
extern "C" {
#endif


QUBFITS_PY_API void init_qubfits();

PyObject* qubfits_LoadPlugin(PyObject* self, PyObject* args);
PyObject* qubfits_CreateCurve(PyObject* self, PyObject* args);
PyObject* qubfits_ListCurves(PyObject* self, PyObject* args);

PyObject* qubfits_CreateFitter(PyObject* self, PyObject* args);
PyObject* qubfits_ListFitters(PyObject* self, PyObject* args);

PyObject* qubfits_UseData(PyObject* self, PyObject* args);
PyObject* qubfits_ChiSqrUncertainty(PyObject* self, PyObject* args);


QUBFITS_PY_API PyObject*    qubfits_curve_wrap(IQUB_Curve* curve, bool owned);
QUBFITS_PY_API int          qubfits_curve_check(PyObject* pyCurve);
QUBFITS_PY_API void         qubfits_curve_free(PyObject* pyCurve);
QUBFITS_PY_API IQUB_Curve*  qubfits_curve_obj(PyObject* pyCurve);
  // py clients shouldn't be creating these:
  //  PyObject* qubfits_curve(PyObject* self, PyObject* args);

PyObject* qubfits_curve_repr(PyObject* self);
PyObject* qubfits_curve_str(PyObject* self);
int       qubfits_curve_compare(PyObject* v, PyObject* w);
PyObject* qubfits_curve_getattr(PyObject* self, char* name);
int       qubfits_curve_setattr(PyObject* self, char* name, PyObject* val);
  // r/w: eqn, componentCount
  // r:   name, canSetEqn, paramCount, hasDeriv, msg, state

PyObject* qubfits_curve_addEvent(PyObject* self, PyObject* args);
PyObject* qubfits_curve_delEvent(PyObject* self, PyObject* args);

PyObject* qubfits_curve_setExtraVars(PyObject* self, PyObject* args);
PyObject* qubfits_curve_getParamName(PyObject* self, PyObject* args);
PyObject* qubfits_curve_getParamVal(PyObject* self, PyObject* args);
PyObject* qubfits_curve_setParamVal(PyObject* self, PyObject* args);
PyObject* qubfits_curve_getParamConst(PyObject* self, PyObject* args);
PyObject* qubfits_curve_setParamConst(PyObject* self, PyObject* args);
PyObject* qubfits_curve_initParams(PyObject* self, PyObject* args);
PyObject* qubfits_curve_evalArrayAll(PyObject* self, PyObject* args);
PyObject* qubfits_curve_evalArrayOne(PyObject* self, PyObject* args);
PyObject* qubfits_curve_evalDerivArray(PyObject* self, PyObject* args);
PyObject* qubfits_curve_updateState(PyObject* self, PyObject* args);

QUBFITS_PY_API PyObject*    qubfits_fitter_wrap(IQUB_Fitter* fitter, bool owned);
QUBFITS_PY_API int          qubfits_fitter_check(PyObject* pyEvts);
QUBFITS_PY_API void         qubfits_fitter_free(PyObject* pyEvts);
QUBFITS_PY_API IQUB_Fitter* qubfits_fitter_obj(PyObject* pyEvts);
  // py clients shouldn't be creating these:
  //  PyObject* qubfits_fitter(PyObject* self, PyObject* args);

PyObject* qubfits_fitter_repr(PyObject* self);
PyObject* qubfits_fitter_str(PyObject* self);
int       qubfits_fitter_compare(PyObject* v, PyObject* w);
PyObject* qubfits_fitter_getattr(PyObject* self, char* name);
  // int       qubfits_fitter_setattr(PyObject* self, char* name, PyObject* val);
  // r: name, paramCount, iterations, curve, msg, state

PyObject* qubfits_fitter_addEvent(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_delEvent(PyObject* self, PyObject* args);

PyObject* qubfits_fitter_getParamName(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_getParamVal(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_setParamVal(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_fit(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_stop(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_getParamErr(PyObject* self, PyObject* args);
PyObject* qubfits_fitter_updateState(PyObject* self, PyObject* args);

#ifdef __cplusplus    /*`__cplusplus' is #defined iff compiler is C++*/
}
#endif


  // from qf_data.cpp
void TreeToQFData(QUB_Tree tree, QF_Data* data, double** ff);
void CleanupQFData(QF_Data* data);
QUB_Tree QF_UseData(QUB_Tree treeData, QUB_Tree use);


#endif
